class LearningObjectsController < ApplicationController
  before_action :set_learning_object, only: [:show, :edit, :update, :destroy, :like, :bookmarks, :collections]
  after_action :increment_learning_object_views, only: [:show]
  before_action :authenticate_user!, except: [:index, :show]
  before_action :set_complaint_messages, only: :show

  # GET /learning_objects
  # GET /learning_objects.json
  def index
    @learning_objects = learning_object_repository.all
  end

  # GET /learning_objects/1
  # GET /learning_objects/1.json
  def show
    @liked = learning_object_repository.liked?(current_user, @learning_object) if user_signed_in?
  end

  # GET /learning_objects/new
  def new
    @learning_object = LearningObject.new
    @school_levels = ['Educação Infantil', 'Ensino Fundamental', 'Ensino Médio']
    @subjects = Subject.default_list
  end

  # GET /learning_objects/1/edit
  def edit
  end

  # POST /learning_objects
  # POST /learning_objects.json
  def create
    collection_struct = Struct.new(:id)
    collection = collection_struct.new('4')
    repo = Dspace::Client.instance.repository.collection_repository
    strategy = DSpaceRest::Strategies::Uploads::CurlStrategy.new(repo.rest_client.url,repo.rest_client.options[:headers][:rest_dspace_token])

    args = {}
    args['name'] = learning_object_params[:name]
    args['type'] = learning_object_params[:type]
    args['last_modified'] = Time.now.to_s
    args['metadata'] = []
    # trata o form do learning object
    params[:learning_object][:date_created] = Time.now
    params[:learning_object][:date_available] = Time.now
    params[:learning_object][:type] = get_file_type params[:learning_object][:file]
    lo = DSpaceRest::Item.new(args)
    dspace_keys = get_dspace_metadata_names("invert")
    params[:learning_object].each do |k,v|
      unless dspace_keys[k].nil?
        if v.kind_of?(Array)
          v.each do |item|
            lo.add_metadata(dspace_keys[k],item,"pt-BR")
          end
        else
          lo.add_metadata(dspace_keys[k], v, "pt-BR")
        end
      end
    end
    response = repo.create_item_for(collection, lo)
    file = params[:learning_object][:file]

    repo = Dspace::Client.instance.repository.item_repository
    bitstream_response = repo.create_bitstream_for(response, file.tempfile.path, strategy)

    subjects = []
    params[:learning_object][:subjects].each do |subject|
      subjects << subject_repository.find_by_name(subject)
    end

    @learning_object = LearningObject.new(learning_object_params)
    @learning_object.type = get_file_type file
    @learning_object.id_dspace = response.id
    @learning_object.subjects = subjects
    @learning_object.created_at = Time.now
    @learning_object.last_modified = Time.now
    @learning_object.publisher = current_user
    @learning_object.metadata = lo.to_h[:metadata].as_json.to_s.gsub! '=>', ':'
    @learning_object.bitstreams = [bitstream_response.as_json]

    respond_to do |format|
      if learning_object_repository.create @learning_object
        learning_object_repository.create_relations @learning_object
        format.html { redirect_to me_users_path, notice: 'Learning object was successfully created.' }
      else
        format.html { render :new }
      end
    end
  end



  # PATCH/PUT /learning_objects/1
  # PATCH/PUT /learning_objects/1.json
  def update
    respond_to do |format|
      if learning_object_repository.update(learning_object_params)
        format.html { redirect_to @learning_object, notice: 'Learning object was successfully updated.' }
      else
        format.html { render :edit }
      end
    end
  end

  # DELETE /learning_objects/1
  # DELETE /learning_objects/1.json
  def destroy
    learning_object_repository.destroy @learning_object

    respond_to do |format|
      format.html { redirect_to learning_objects_url, notice: 'Learning object was successfully destroyed.' }
    end
  end

  def report_object
    learning_object_repository.report current_user, @learning_object, message, description
  end

  # POST /learning_objects/1/like
  def like
    if @learning_object.liked? current_user
      @learning_object.dislike current_user
    else
      @learning_object.like current_user
    end

    if request.xhr?
      render json: {count: @learning_object.likes, id: params[:id]}
    end
  end

  # POST /learning_objects/1/bookmarks
  def bookmarks
    bookmarks = current_user.bookmarks
    bookmarks.add @learning_object
    collection_repository.save_learning_objects bookmarks

    if request.xhr?
      render json: {id: params[:id]}
    end
  end

  # GET /learning_objects/1/collections.json
  def collections
    @collections = @learning_object.collections
  end

  private

  def get_file_type(file)
    type = file.content_type.split('/').first
    case type
    when 'video'
      "Vídeo"
    when 'image'
      "Imagem"
    when 'audio'
      "Áudio"
    else
      "Outros"
    end
  end

  def get_dspace_metadata_names(invert=nil)
    h = {




        "dc.contributor.author" => "author",
        "dc.date.accessioned" => "date_created",
        "dc.date.available" => "date_available",
        #dc.identifier.uri
        "dc.language" => "language",
        "dc.title" => "name",
        "dc.type" => "type",
        "dc.subject.category" => "subjects"
        # "dc.description" => "description",
        # "dc.source" => "source",
        # "dc.date.issued" => "date",
        # "dc.rights.license" => "license",
        # "dc.rights.holder" => "copyright",
        # "dc.location.country" => "country",
        # "dc.date.accessioned" => "date_created",
        # "dc.date.available" => "date_available",
        # "dc.subject.keyword" => "keywords",
        # "dc.date.issued" => "date_issued",
        # "dc.date.submitted" => "date_submitted",
        # "dc.publisher" => "publisher"
        # TODO: dc.audience.mediator

    }
    if (invert != nil)
      return h.invert
    end
    h
  end

  # Use callbacks to share common setup or constraints between actions.
  def set_learning_object
    @learning_object = learning_object_repository.find params[:id]
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def learning_object_params
    params[:learning_object].permit(:author, :name, :type, :description, :subjects, :school_level)
  end

  def increment_learning_object_views
    if user_signed_in?
      learning_object_repository.increment_views current_user, @learning_object
    end
  end

  def set_complaint_messages
    @complaint = Complaint.new
    @messages = [
        Complaint.copyrights,
        Complaint.ofensive_content,
        Complaint.ofensive_user,
        Complaint.fake_user
    ]
  end

end
